@@ -285,6 +285,9 @@ namespace xt
         const_reverse_iterator crbegin() const noexcept;
         const_reverse_iterator crend() const noexcept;
 
+        derived_type& derived_cast() noexcept;
+        const derived_type& derived_cast() const noexcept;
+
     protected:
 
         xbuffer_adaptor_base() = default;
@@ -295,11 +298,6 @@ namespace xt
 
         xbuffer_adaptor_base(self_type&&) = default;
         self_type& operator=(self_type&&) = default;
-
-    private:
-
-        derived_type& derived_cast() noexcept;
-        const derived_type& derived_cast() const noexcept;
     };
 
     template <class D>
@@ -372,10 +370,7 @@ namespace xt
         using const_reverse_iterator = typename buffer_base_type::const_reverse_iterator;
         using temporary_type = uvector<value_type, allocator_type>;
 
-        xbuffer_adaptor() = default;
-
-        template <class P, class AT = allocator_type>
-        xbuffer_adaptor(P&& data, size_type size, AT&& alloc = allocator_type());
+        using base_type::base_type;
 
         ~xbuffer_adaptor() = default;
 
@@ -716,7 +711,7 @@ namespace xt
     template <class D>
     inline bool xbuffer_adaptor_base<D>::empty() const noexcept
     {
-        return derived_cast().size() != size_type(0);
+        return derived_cast().size() == size_type(0);
     }
 
     template <class D>
@@ -843,7 +838,7 @@ namespace xt
     inline bool operator==(const xbuffer_adaptor_base<D>& lhs,
                            const xbuffer_adaptor_base<D>& rhs)
     {
-        return lhs.size() == rhs.size() && std::equal(lhs.begin(), lhs.end(), rhs.begin());
+        return lhs.derived_cast().size() == rhs.derived_cast().size() && std::equal(lhs.begin(), lhs.end(), rhs.begin());
     }
 
     template <class D>
@@ -893,13 +888,6 @@ namespace xt
      * xbuffer_adaptor implementation *
      **********************************/
 
-    template <class CP, class O, class A>
-    template <class P, class AT>
-    inline xbuffer_adaptor<CP, O, A>::xbuffer_adaptor(P&& data, size_type size, AT&& alloc)
-        : base_type(std::forward<P>(data), size, std::forward<AT>(alloc))
-    {
-    }
-
     template <class CP, class O, class A>
     inline auto xbuffer_adaptor<CP, O, A>::operator=(temporary_type&& tmp) -> self_type&
     {
